function [x,iter,nf] = NCG(func,x0,tol,maxit,varargin);

x = x0; nf = 0; ng = 0;
[f,g] = feval(func,x,varargin{:});
nf = nf + 1;
p = -g; gnrm = norm(g);

for iter = 0:maxit;
    
    crit = gnrm / (1 + abs(f));
    if mod(iter,10) == 0 || crit < tol;
        fprintf('iter: %5i f = %9.4e, norm(g) = %6.2e, crit = %6.2e\n',...
             iter,f,gnrm,crit);
    end
    if crit < tol; break; end

    g0 = g;
    [a,nf] = linesearch(func,x,f,g,p,nf,varargin{:}); 
    x = x + a*p;
    [f,g] = feval(func,x,varargin{:});
    nf = nf + 1;
    
    gnrm0 = gnrm; gnrm = norm(g);
    b = gnrm^2/gnrm0^2;        % CG-FR 
    b = b - g0'*g/gnrm0^2;     % CG-PR
    %b = max(b,0);              % CG-PR+
    %b = 0;                     % Steepest Descent
    p = -g + b*p;

end;


function [a,nf] = linesearch(func,x,f,g,p,nf,varargin);

    c1 = 1.e-4; rho = .25; a = 1; gtp = g'*p;
    if gtp > 0; p = -g; gtp = -g'*g; end
    dec = c1*gtp; amin = sqrt(eps);
    
    while a > amin;
        xt = x + a*p;
        fnew = feval(func,xt,varargin{:});
        nf = nf + 1;
        if fnew < f + a*dec; break; end
        a = rho*a;
        if a < amin; break; end
    end;
