function [f,g] = TVL2(u,h,mu,tau)

% Compute the TVL2 function (with pertubation tau): 
%          f = TV(u) + .5*mu*||u-h||^2
% and its gradient g, where u = vec(U) and U is m x m.

m = sqrt(length(u));
zr = zeros(1,m);
zc = zeros(m,1);
U = reshape(u,m,m);
Ux = diff([zc U],1,2);
Uy = diff([zr; U]);

fidelity = u - h;
U = sqrt(Ux.^2 + Uy.^2 + tau);
f =  sum(U(:)) + .5*mu*norm(fidelity)^2;

if nargout < 2; return; end

Ux = Ux./U; 
Uy = Uy./U;
U = -diff([Ux zc],1,2);
U = U - diff([Uy; zr]);
g = U(:) + mu*fidelity;
